@@ -60,6 +60,15 @@ class UsernameOrEmailAuthenticationForm(AuthenticationForm):
     password = forms.CharField(widget=forms.PasswordInput(
         attrs={'placeholder': 'Password'}), label=_("Password"))
 
+    def __init__(self, *args, **kwargs):
+        super(UsernameOrEmailAuthenticationForm, self).__init__(*args, **kwargs)
+        username_label = self.initial.get('username_label', None)
+        if username_label:
+            placeholder_label = '%s or Email' % username_label
+            self.fields['username'].label = placeholder_label
+            self.fields['username'].widget.attrs['placeholder'] \
+                = placeholder_label
+
 
 class UsernameOrEmailModelBackend(object):
     """
@@ -70,15 +79,18 @@ class UsernameOrEmailModelBackend(object):
 
     def authenticate(self, username=None, password=None):
         if '@' in username:
-            kwargs = {'email': username}
+            kwargs = {'email__iexact': username}
         else:
-            kwargs = {'username': username}
+            kwargs = {'username__iexact': username}
         try:
-            user = User.objects.get(**kwargs) #pylint: disable=star-args
+            user = User.objects.get(**kwargs)
             if user.check_password(password):
                 return user
         except User.DoesNotExist:
-            return None
+            # Run the default password hasher once to reduce the timing
+            # difference between an existing and a non-existing user (#20760).
+            User().set_password(password)
+        return None
 
     def get_user(self, user_id):
         try:
