@@ -1,4 +1,4 @@
-# Copyright (c) 2014, Djaodjin Inc.
+# Copyright (c) 2017, Djaodjin Inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -26,17 +26,18 @@
 Decorators that check a User a verified email address.
 """
 
-import urlparse
-
 from functools import wraps
+
 from django.contrib import messages
 from django.contrib.auth import REDIRECT_FIELD_NAME, logout as auth_logout
-from django.contrib.sites.models import RequestSite, Site
+from django.core.urlresolvers import reverse
 from django.utils.decorators import available_attrs
+from django.utils import six
 from django.utils.translation import ugettext_lazy as _
 
-from signup.backends import get_email_backend
-from signup import settings
+from . import settings, signals
+from .models import EmailContact
+from .utils import has_invalid_password
 
 
 def _insert_url(request, redirect_field_name=REDIRECT_FIELD_NAME,
@@ -48,8 +49,9 @@ def _insert_url(request, redirect_field_name=REDIRECT_FIELD_NAME,
     path = request.build_absolute_uri()
     # If the login url is the same scheme and net location then just
     # use the path as the "next" url.
-    login_scheme, login_netloc = urlparse.urlparse(inserted_url)[:2]
-    current_scheme, current_netloc = urlparse.urlparse(path)[:2]
+    login_scheme, login_netloc = six.moves.urllib.parse.urlparse(
+        inserted_url)[:2]
+    current_scheme, current_netloc = six.moves.urllib.parse.urlparse(path)[:2]
     if ((not login_scheme or login_scheme == current_scheme) and
         (not login_netloc or login_netloc == current_netloc)):
         path = request.get_full_path()
@@ -57,7 +59,7 @@ def _insert_url(request, redirect_field_name=REDIRECT_FIELD_NAME,
     return redirect_to_login(path, inserted_url, redirect_field_name)
 
 
-def _send_verification_email(user, site,
+def send_verification_email(email_contact, request,
                            next_url=None,
                            redirect_field_name=REDIRECT_FIELD_NAME):
     """
@@ -67,12 +69,14 @@ def _send_verification_email(user, site,
     the verification email was sent from so that the user stays on her
     workflow once verification is completed.
     """
-    get_email_backend().send([user.email],
-        'accounts/verification.eml',
-        {'user': user, 'site': site,
-         'verification_key': user.email_verification_key,
-         'expiration_days': settings.KEY_EXPIRATION,
-         redirect_field_name: next_url})
+    back_url = request.build_absolute_uri(reverse('registration_activate',
+        args=(email_contact.verification_key,)))
+    if next_url:
+        back_url += '?%s=%s' % (redirect_field_name, next_url)
+    signals.user_verification.send(
+        sender=__name__, user=email_contact.user, request=request,
+        back_url=back_url, expiration_days=settings.KEY_EXPIRATION)
+    messages.info(request, "Activation e-mail sent.")
 
 
 # The user we are looking to activate might be different from
@@ -84,17 +88,15 @@ def check_user_active(request, user,
     Checks that a *user* is active. We won't activate the account of
     a user until we checked the email address is valid.
     """
-    if user.has_invalid_password:
+    if has_invalid_password(user):
         # Let's send e-mail again.
-        if not user.is_reachable:
-            if Site._meta.installed: #pylint: disable=protected-access
-                site = Site.objects.get_current()
-            else:
-                site = RequestSite(request)
+        first_unverified_email = EmailContact.objects.unverified_for_user(
+            user).first()
+        if first_unverified_email is not None:
             if not next_url:
                 next_url = request.META['PATH_INFO']
-            _send_verification_email(
-                user, site, next_url=next_url,
+            send_verification_email(
+                first_unverified_email, request, next_url=next_url,
                 redirect_field_name=redirect_field_name)
             return False
     return True
