@@ -1,4 +1,4 @@
-# Copyright (c) 2014, Djaodjin Inc.
+# Copyright (c) 2017, Djaodjin Inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -24,12 +24,14 @@
 
 """Forms for the signup app"""
 
+from captcha.fields import ReCaptchaField
 from django import forms
 from django.contrib.auth.forms import (
     PasswordResetForm as PasswordResetBaseForm, SetPasswordForm)
 from django.utils.translation import ugettext_lazy as _
 
-from signup.compat import User
+from . import settings
+from .compat import User
 
 #pylint: disable=old-style-class,no-init
 
@@ -41,18 +43,25 @@ class NameEmailForm(forms.Form):
     """
     full_name = forms.RegexField(
         regex=r'^[\w\s]+$', max_length=60,
-        widget=forms.TextInput(attrs={'placeholder':'Full Name'}),
-        label=_("Full Name"),
+        widget=forms.TextInput(attrs={'placeholder':'Full name'}),
+        label=_("Full name"),
         error_messages={'invalid':
             _("Sorry we do not recognize some characters in your full name.")})
     email = forms.EmailField(
         widget=forms.TextInput(attrs={'placeholder':'Email',
                                       'maxlength': 75}),
-        label=_("E-mail"))
+        label=_("Email address"))
+
+    def __init__(self, *args, **kwargs):
+        super(NameEmailForm, self).__init__(*args, **kwargs)
+        if settings.REQUIRES_RECAPTCHA:
+            self.fields['captcha'] = ReCaptchaField()
 
 
 class PasswordChangeForm(SetPasswordForm):
 
+    submit_title = 'Update'
+
     def __init__(self, *args, **kwargs):
         user = kwargs.pop('instance')
         super(PasswordChangeForm, self).__init__(user, *args, **kwargs)
@@ -67,6 +76,7 @@ class UserForm(forms.ModelForm):
     """
     Form to update a ``User`` profile.
     """
+    submit_title = 'Update'
 
     username = forms.CharField(widget=forms.TextInput(
         attrs={'placeholder': 'Username'}),
