@@ -1,4 +1,4 @@
-# Copyright (c) 2014, Djaodjin Inc.
+# Copyright (c) 2017, Djaodjin Inc.
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -24,45 +24,38 @@
 
 """Extra Forms and Views that might prove useful to register users."""
 
+import logging
+
 from django.core.urlresolvers import reverse
 from django.contrib import messages
 from django.contrib.auth import login as auth_login, logout as auth_logout
-
 from django.contrib.auth import REDIRECT_FIELD_NAME, authenticate
 from django.contrib.auth.decorators import login_required
 from django.contrib.auth.tokens import default_token_generator
-from django.contrib.sites.models import get_current_site
 from django.http import HttpResponseRedirect
 from django.shortcuts import redirect
 from django.template.response import TemplateResponse
-from django.utils.decorators import method_decorator
 from django.utils.encoding import force_bytes
 from django.utils.http import urlsafe_base64_decode, urlsafe_base64_encode
 from django.utils.safestring import mark_safe
 from django.utils.translation import ugettext_lazy as _
-from django.views.decorators.debug import sensitive_post_parameters
-from django.views.decorators.cache import never_cache
+from django.views.decorators.cache import add_never_cache_headers
 from django.views.generic.base import ContextMixin, TemplateResponseMixin, View
 from django.views.generic.detail import BaseDetailView
 from django.views.generic.edit import FormMixin, ProcessFormView, UpdateView
 
-from signup.auth import validate_redirect
-from signup.backends import get_email_backend
-from signup.decorators import check_user_active, _send_verification_email
-from signup.compat import User
-from signup.forms import (NameEmailForm, PasswordChangeForm, PasswordResetForm,
+from . import settings, signals
+from .auth import validate_redirect
+from .backends.auth import UsernameOrEmailAuthenticationForm
+from .compat import User
+from .decorators import check_user_active, send_verification_email
+from .forms import (NameEmailForm, PasswordChangeForm, PasswordResetForm,
     UserForm)
-from signup.backends.auth import UsernameOrEmailAuthenticationForm
-from signup import signals
-from signup import settings
+from .models import EmailContact
+from .utils import has_invalid_password
 
 
-def _redirect_to(url):
-    try:
-        to_url, args, kwargs = url #(cast to tuple) pylint: disable=star-args
-        return redirect(to_url, *args, **kwargs) #pylint: disable=star-args
-    except ValueError:
-        return redirect(url)
+LOGGER = logging.getLogger(__name__)
 
 
 class RedirectFormMixin(FormMixin):
@@ -82,6 +75,45 @@ class RedirectFormMixin(FormMixin):
         return context
 
 
+class AuthTemplateResponseMixin(TemplateResponseMixin):
+    """
+    Returns a *disabled* page regardless when DISABLED_AUTHENTICATION
+    is True.
+    """
+
+    def get_context_data(self, **kwargs):
+        context = super(AuthTemplateResponseMixin, self).get_context_data(
+            **kwargs)
+        # URLs for user
+        user_urls = {}
+        if not self.request.user.is_authenticated():
+            user_urls = {
+               'login': reverse('login'),
+               'password_reset': reverse('password_reset'),
+               'register': reverse('registration_register'),
+            }
+        if 'urls' in context:
+            if 'user' in context['urls']:
+                context['urls']['user'].update(user_urls)
+            else:
+                context['urls'].update({'user': user_urls})
+        else:
+            context.update({'urls': {'user': user_urls}})
+        return context
+
+    def dispatch(self, request, *args, **kwargs):
+        if request.method.lower() in self.http_method_names:
+            if settings.DISABLED_AUTHENTICATION:
+                context = {}
+                response_kwargs = {}
+                response_kwargs.setdefault('content_type', self.content_type)
+                return TemplateResponse(
+                    request=request, template='accounts/disabled.html',
+                    context=context, **response_kwargs)
+        return super(AuthTemplateResponseMixin, self).dispatch(
+            request, *args, **kwargs)
+
+
 class RedirectFormView(RedirectFormMixin, ProcessFormView):
     """
     Redirects on form valid.
@@ -102,19 +134,16 @@ class PasswordResetBaseView(RedirectFormMixin, ProcessFormView):
             if user.is_active and user.has_usable_password():
                 # Make sure that no email is sent to a user that actually has
                 # a password marked as unusable
-                site = get_current_site(self.request)
                 uid = urlsafe_base64_encode(force_bytes(user.pk))
                 token = self.token_generator.make_token(user)
                 back_url = self.request.build_absolute_uri(
                     reverse('password_reset_confirm', args=(uid, token)))
-                get_email_backend().send([user.email],
-                    'accounts/password_reset.eml',
-                    {'user': user, 'site': site,
-                     'back_url': back_url,
-                     'expiration_days': settings.KEY_EXPIRATION,
-                     # XXX Reason for a redirect after password reset?
-                     # redirect_field_name: next_url
-                     })
+                next_url = validate_redirect(self.request)
+                if next_url:
+                    back_url += '?%s=%s' % (REDIRECT_FIELD_NAME, next_url)
+                signals.user_reset_password.send(
+                    sender=__name__, user=user, request=self.request,
+                    back_url=back_url, expiration_days=settings.KEY_EXPIRATION)
         return super(PasswordResetBaseView, self).form_valid(form)
 
     def get_success_url(self):
@@ -152,6 +181,8 @@ class PasswordResetConfirmBaseView(RedirectFormMixin, ProcessFormView):
                                     user, self.kwargs.get('token')):
             if form.is_valid():
                 form.save()
+                LOGGER.info("%s reset her/his password.", self.request.user,
+                    extra={'event': 'resetpassword', 'request': request})
                 return self.form_valid(form)
         return self.form_invalid(form)
 
@@ -182,52 +213,67 @@ class SignupBaseView(RedirectFormMixin, ProcessFormView):
     form_class = NameEmailForm
     fail_url = ('registration_register', (), {})
 
+    @staticmethod
+    def first_and_last_names(**cleaned_data):
+        first_name = cleaned_data.get('first_name', None)
+        last_name = cleaned_data.get('last_name', None)
+        if not first_name:
+            # If the form does not contain a first_name/last_name pair,
+            # we assume a full_name was passed instead.
+            full_name = cleaned_data['full_name']
+            name_parts = full_name.split(' ')
+            if len(name_parts) > 0:
+                first_name = name_parts[0]
+                last_name = ' '.join(name_parts[1:])
+            else:
+                first_name = full_name
+                last_name = ''
+        return first_name, last_name
+
+    def dispatch(self, request, *args, **kwargs):
+        if request.method.lower() in self.http_method_names:
+            if settings.DISABLED_REGISTRATION:
+                context = {}
+                response_kwargs = {}
+                response_kwargs.setdefault('content_type', self.content_type)
+                return TemplateResponse(request=request,
+                    template='accounts/disabled_registration.html',
+                    context=context, **response_kwargs)
+        return super(SignupBaseView, self).dispatch(request, *args, **kwargs)
+
     def form_valid(self, form):
         new_user = self.register(**form.cleaned_data)
         if new_user:
             success_url = self.get_success_url()
         else:
             success_url = self.request.META['PATH_INFO']
-        return _redirect_to(success_url)
+        return HttpResponseRedirect(success_url)
 
     def register(self, **cleaned_data):
         #pylint: disable=maybe-no-member
-        full_name, email = cleaned_data['full_name'], cleaned_data['email']
+        email = cleaned_data['email']
         users = User.objects.filter(email=email)
         if users.exists():
             user = users.get()
             if check_user_active(self.request, user,
                                  next_url=self.get_success_url()):
-                messages.info(self.request, mark_safe(_(
+                messages.warning(self.request, mark_safe(_(
                     'This email address has already been registered!'\
 ' Please <a href="%s">login</a> with your credentials. Thank you.'
                     % reverse('login'))))
             else:
-                messages.info(self.request, mark_safe(_(
+                messages.warning(self.request, mark_safe(_(
                     "This email address has already been registered!"\
 " You should now secure and activate your account following "\
 " the instructions we just emailed you. Thank you.")))
             return None
 
-        name_parts = full_name.split(' ')
-        if len(name_parts) > 0:
-            first_name = name_parts[0]
-            last_name = ' '.join(name_parts[1:])
-        else:
-            first_name = full_name
-            last_name = ''
-        username = None
-        if cleaned_data.has_key('username'):
-            username = cleaned_data['username']
-        if username and cleaned_data.has_key('password'):
-            user = User.objects.create_user(
-                username=username, password=cleaned_data['password'],
-                email=email, first_name=first_name, last_name=last_name)
-        else:
-            user = User.objects.create_inactive_user(email,
-                username=username, first_name=first_name, last_name=last_name)
-        signals.user_registered.send(
-            sender=__name__, user=user, request=self.request)
+        first_name, last_name = self.first_and_last_names(**cleaned_data)
+        username = cleaned_data.get('username', None)
+        password = cleaned_data.get('new_password1', None)
+        user = User.objects.create_user(username,
+            email=email, password=password,
+            first_name=first_name, last_name=last_name)
 
         # Bypassing authentication here, we are doing frictionless registration
         # the first time around.
@@ -243,22 +289,25 @@ class ActivationBaseView(ContextMixin, View):
     """
     http_method_names = ['get']
     token_generator = default_token_generator
+    key_url_kwarg = 'verification_key'
 
     def get(self, request, *args, **kwargs):
-        verification_key = kwargs['verification_key']
+        verification_key = self.kwargs.get(self.key_url_kwarg)
         #pylint: disable=maybe-no-member
-        user = User.objects.find_user(verification_key)
+        user = EmailContact.objects.find_user(verification_key)
         if user:
-            if user.has_invalid_password:
+            if has_invalid_password(user):
                 messages.info(self.request,
-                    _("Please set a password to protect your account."))
-                url = reverse('registration_password_confirm',
-                              args=(user.email_verification_key,
-                                    self.token_generator.make_token(user)))
+                    _("You are about to activate your account. Please set"\
+                      " a password to secure it."))
+                url = reverse('registration_password_confirm', args=(
+                    verification_key, self.token_generator.make_token(user)))
             else:
-                user = User.objects.activate_user(verification_key)
-                signals.user_activated.send(
-                    sender=__name__, user=user, request=self.request)
+                user = EmailContact.objects.activate_user(verification_key)
+                # XXX Should we directly login user here?
+                signals.user_activated.send(sender=__name__,
+                    user=user, verification_key=verification_key,
+                    request=self.request)
                 messages.info(self.request,
                     _("Thank you. Your account is now active." \
                           " You can sign in at your convienience."))
@@ -268,8 +317,9 @@ class ActivationBaseView(ContextMixin, View):
                 success_url = "%s?%s=%s" % (url, REDIRECT_FIELD_NAME, next_url)
             else:
                 success_url = url
-            return _redirect_to(success_url)
-        return super(ActivationBaseView, self).get(request, *args, **kwargs)
+            return HttpResponseRedirect(success_url)
+        context = self.get_context_data(**kwargs)
+        return self.render_to_response(context)
 
 
 class SendActivationView(BaseDetailView):
@@ -281,11 +331,15 @@ class SendActivationView(BaseDetailView):
 
     def get(self, request, *args, **kwargs):
         user = self.get_object()
-        site = get_current_site(request)
-        _send_verification_email(user, site)
-        messages.info(self.request, "Activation e-mail sent.")
+        unverified_email = EmailContact.objects.unverified_for_user(
+            self.get_object()).first()
+        if unverified_email is not None:
+            send_verification_email(unverified_email, request)
+        else:
+            messages.info(self.request,
+                _("This email address has already been verified."))
         return HttpResponseRedirect(
-            reverse('users_profile', args=(self.object,)))
+            reverse('users_profile', args=(user,)))
 
 
 class SigninBaseView(RedirectFormMixin, ProcessFormView):
@@ -297,6 +351,8 @@ class SigninBaseView(RedirectFormMixin, ProcessFormView):
 
     def form_valid(self, form):
         auth_login(self.request, form.get_user())
+        LOGGER.info("%s signed in.", self.request.user,
+            extra={'event': 'login', 'request': self.request})
         return super(SigninBaseView, self).form_valid(form)
 
 
@@ -306,14 +362,20 @@ class SignoutBaseView(RedirectFormMixin, View):
     """
 
     def get(self, request, *args, **kwargs):
+        LOGGER.info("%s signed out.", self.request.user,
+            extra={'event': 'logout', 'request': request})
         auth_logout(request)
         next_url = self.get_success_url()
-        if next_url:
-            return HttpResponseRedirect(next_url)
-        return super(SignoutBaseView, self).get(request, *args, **kwargs)
+        response = HttpResponseRedirect(next_url)
+        if settings.LOGOUT_CLEAR_COOKIES:
+            for cookie in settings.LOGOUT_CLEAR_COOKIES:
+                response.delete_cookie(cookie)
+        return response
 
 
-class UserProfileView(UpdateView):
+# Actual views to instantiate start here:
+
+class UserProfileView(AuthTemplateResponseMixin, UpdateView):
     """
     If a user is manager for an Organization, she can access the Organization
     profile. If a user is manager for an Organization subscribed to another
@@ -326,6 +388,26 @@ class UserProfileView(UpdateView):
     slug_url_kwarg = 'user'
     template_name = 'users/user_form.html'
 
+    def get_context_data(self, **kwargs):
+        context = super(UserProfileView, self).get_context_data(**kwargs)
+        # URLs for user
+        if self.request.user.is_authenticated():
+            user_urls = {
+                'password_change': reverse(
+                    'password_change', args=(self.object,)),
+                'profile': reverse('users_profile', args=(self.object,)),
+                'profile_redirect': reverse('accounts_profile')
+            }
+        if 'urls' in context:
+            if 'user' in context['urls']:
+                context['urls']['user'].update(user_urls)
+            else:
+                context['urls'].update({'user': user_urls})
+        else:
+            context.update({'urls': {'user': user_urls}})
+        setattr(context['user'], 'full_name', context['user'].get_full_name())
+        return context
+
     def get_success_url(self):
         messages.info(self.request, 'Profile Updated.')
         return reverse('users_profile', args=(self.object,))
@@ -340,6 +422,10 @@ class PasswordChangeView(UserProfileView):
     template_name = 'users/password_change_form.html'
 
     def get_success_url(self):
+        LOGGER.info("%s updated password for %s.",
+            self.request.user, self.object, extra={
+            'event': 'update-password', 'request': self.request,
+            'modified': self.object.username})
         messages.info(self.request, "Password has been updated successfuly.")
         return reverse('users_profile', args=(self.object,))
 
@@ -352,12 +438,16 @@ class RegistrationPasswordConfirmBaseView(RedirectFormMixin, ProcessFormView):
     """
     form_class = PasswordChangeForm
     token_generator = default_token_generator
+    key_url_kwarg = 'verification_key'
 
-    @method_decorator(sensitive_post_parameters)
-    @method_decorator(never_cache)
     def dispatch(self, request, *args, **kwargs):
-        return super(RegistrationPasswordConfirmBaseView, self).dispatch(
+        # We put the code inline instead of using method_decorator() otherwise
+        # kwargs is interpreted as parameters in sensitive_post_parameters.
+        request.sensitive_post_parameters = '__ALL__'
+        response = super(RegistrationPasswordConfirmBaseView, self).dispatch(
             request, *args, **kwargs)
+        add_never_cache_headers(response)
+        return response
 
     def post(self, request, *args, **kwargs):
         """
@@ -370,18 +460,22 @@ class RegistrationPasswordConfirmBaseView(RedirectFormMixin, ProcessFormView):
         # to make sure both the *verification_key* and *token* are valid
         # before doing any modification of the underlying models.
         #pylint: disable=maybe-no-member
-        user = User.objects.find_user(self.kwargs.get('verification_key'))
-        if user is not None and self.token_generator.check_token(user,
-                                                   self.kwargs.get('token')):
+        user = self.object
+        if (user is not None
+            and self.token_generator.check_token(
+                user, self.kwargs.get('token'))):
             if form.is_valid():
                 return self.form_valid(form)
         return self.form_invalid(form)
 
     def form_valid(self, form):
         #pylint: disable=maybe-no-member
-        user = User.objects.activate_user(self.kwargs.get('verification_key'))
-        signals.user_activated.send(
-            sender=__name__, user=user, request=self.request)
+        self.object = form.save() # If we don't save the ``User`` model here,
+                                  # we won't be able to authenticate later.
+        verification_key = self.kwargs.get('verification_key')
+        user = EmailContact.objects.activate_user(verification_key)
+        signals.user_activated.send(sender=__name__,
+            user=user, verification_key=verification_key, request=self.request)
         messages.info(self.request,
                       _("Thank you. Your account is now active."))
 
@@ -392,8 +486,7 @@ class RegistrationPasswordConfirmBaseView(RedirectFormMixin, ProcessFormView):
         auth_login(self.request, user_with_backend)
         if self.request.session.test_cookie_worked():
             self.request.session.delete_test_cookie()
-
-        return super(RegistrationPasswordConfirmBaseView, self).form_valid(form)
+        return HttpResponseRedirect(self.get_success_url())
 
     def get_context_data(self, **kwargs):
         context = super(RegistrationPasswordConfirmBaseView,
@@ -408,37 +501,15 @@ class RegistrationPasswordConfirmBaseView(RedirectFormMixin, ProcessFormView):
         """
         Returns the keyword arguments for instantiating the form.
         """
-        kwargs = super(RegistrationPasswordConfirmBaseView,
-                       self).get_form_kwargs()
-        try:
-            uid = urlsafe_base64_decode(self.kwargs.get('uidb64'))
-            self.object = User.objects.get(pk=uid)
-        except (TypeError, ValueError, OverflowError, User.DoesNotExist):
-            self.object = None
+        kwargs = super(
+            RegistrationPasswordConfirmBaseView, self).get_form_kwargs()
+        #pylint: disable=no-member
+        self.object = EmailContact.objects.find_user(
+            self.kwargs.get(self.key_url_kwarg))
         kwargs.update({'instance': self.object})
         return kwargs
 
 
-class AuthTemplateResponseMixin(TemplateResponseMixin):
-    """
-    Returns a *disabled* page regardless when DISABLED_AUTHENTICATION
-    is True.
-    """
-
-    def dispatch(self, request, *args, **kwargs):
-        if request.method.lower() in self.http_method_names:
-            if settings.DISABLED_AUTHENTICATION:
-                context = {}
-                response_kwargs = {}
-                response_kwargs.setdefault('content_type', self.content_type)
-                #pylint:disable=star-args
-                return TemplateResponse(
-                    request=request, template='accounts/disabled.html',
-                    context=context, **response_kwargs)
-        return super(AuthTemplateResponseMixin, self).dispatch(
-            request, *args, **kwargs)
-
-
 class ActivationView(AuthTemplateResponseMixin, ActivationBaseView):
 
     template_name = 'accounts/activate.html'
@@ -466,7 +537,7 @@ class SigninView(AuthTemplateResponseMixin, SigninBaseView):
     template_name = 'accounts/login.html'
 
 
-class SignoutView(TemplateResponseMixin, SignoutBaseView):
+class SignoutView(AuthTemplateResponseMixin, SignoutBaseView):
 
     template_name = 'accounts/logout.html'
 
